@@ -3,7 +3,6 @@
 
 #include <SRMTypes.h>
 #include <xf86drmMode.h>
-#include <time.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -133,7 +132,7 @@ SRMDevice *srmConnectorGetDevice(SRMConnector *connector);
  * This function returns the device responsible for rendering operations for the connector.
  *
  * When the **ITSELF** rendering mode is used, this device is the same as the one obtained with srmConnectorGetDevice().
- * However, in the **DUMB** or **CPU** modes, it differs.
+ * However, in the **PRIME**, **DUMB** or **CPU** modes, it differs.
  *
  * @param connector Pointer to the @ref SRMConnector for which you want to obtain the renderer device.
  * @return Pointer to the @ref SRMDevice responsible for rendering operations for the connector.
@@ -513,9 +512,9 @@ UInt32 srmConnectorGetCurrentBufferIndex(SRMConnector *connector);
  * @brief Returns the number of framebuffers.
  *
  * This function returns the number of framebuffers available for the given @ref SRMConnector. 
- * The count may be 1, 2, or 3, depending on the configuration (e.g., single, double or triple buffering).
+ * The count may be 2 or 3, depending on the configuration (double or triple buffering).
  *
- * The number of framebuffers can be modified by setting the **SRM_RENDER_MODE_{ITSELF, DUMB, CPU}_FB_COUNT** environment variables.
+ * The number of framebuffers can be modified by setting the **SRM_RENDER_MODE_{ITSELF, PRIME, DUMB, CPU}_FB_COUNT** environment variables.
  *
  * @param connector Pointer to the @ref SRMConnector to query for the number of framebuffers.
  * @return The number of framebuffers.
@@ -617,7 +616,7 @@ UInt64 srmConnectorGetGammaSize(SRMConnector *connector);
 /**
  * @brief Sets the gamma correction curves for each RGB component.
  *
- * This method allows you to individually set the gamma correction curves for each RGB component.
+ * This method allows you to set the gamma correction curves for each RGB component.
  * The number of elements for each curve (N) should be obtained using srmConnectorGetGammaSize().
  * The table array should then have a size of `3 * N * sizeof(UInt16)` bytes, with N @ref UInt16 values for red,
  * N for green, and N for blue, in that order. Each value of the curves can represent the full range of @ref UInt16.
@@ -625,6 +624,8 @@ UInt64 srmConnectorGetGammaSize(SRMConnector *connector);
  * @note This method must only be called after the connector is initialized.
  *       If the connector is uninitialized or the driver does not support gamma correction,
  *       the function returns 0.
+ *
+ * The default gamma curves are linear.
  * 
  * @param connector Pointer to the @ref SRMConnector.
  * @param table Pointer to the array containing RGB curves for gamma correction.
@@ -723,6 +724,49 @@ clockid_t srmConnectorGetPresentationClock(SRMConnector *connector);
  */
 const SRMPresentationTime *srmConnectorGetPresentationTime(SRMConnector *connector);
 
+/**
+ * @brief Sets a hint of the content type being displayed.
+ *
+ * @see @ref SRM_CONNECTOR_CONTENT_TYPE
+ *
+ * @note The default value is @ref SRM_CONNECTOR_CONTENT_TYPE_GRAPHICS
+ *
+ * @param connector A pointer to the @ref SRMConnector instance.
+ * @param contentType The content type hint.
+ */
+void srmConnectorSetContentType(SRMConnector *connector, SRM_CONNECTOR_CONTENT_TYPE contentType);
+
+/**
+ * @brief Gets the content type hint.
+ *
+ * @see srmConnectorSetContentType()
+ *
+ * @param connector A pointer to the @ref SRMConnector instance.
+ */
+SRM_CONNECTOR_CONTENT_TYPE srmConnectorGetContentType(SRMConnector *connector);
+
+/**
+ * @brief Sets a custom scanout buffer for the primary plane.
+ *
+ * This function allows you to set a custom scanout buffer for the primary plane only during a single frame.
+ * It must called within a `paintGL()` event. Calling it outside a `paintGL()` will result in an error.
+ *
+ * If successfully set, the current buffer index is not updated, and no OpenGL rendering operations should be performed within the `paintGL()` event.
+ * If not set again in subsequent frames, the connector's framebuffers are restored, and the current buffer index continues to be updated as usual.
+ *
+ * @note The size of the buffer must match the dimensions of the current connector's mode.
+ *
+ * If successfully set, the internal reference counter of the buffer is increased, ensuring it remains scannable for at least the given frame
+ * even if it is destroyed.
+ * 
+ * @note If `SRM_DISABLE_CUSTOM_SCANOUT` is set to 1, this function always return 0.
+ *
+ * @param connector A pointer to the @ref SRMConnector instance.
+ * @param buffer The buffer to scan, or `NULL` to restore the default connector framebuffers.
+ * @return 1 if the custom buffer will be scanned, 0 otherwise.
+ */
+UInt8 srmConnectorSetCustomScanoutBuffer(SRMConnector *connector, SRMBuffer *buffer);
+
 /**
  * @}
  */
