@@ -36,10 +36,12 @@ SRMCore *srmCoreCreate(SRMInterface *interface, void *userData)
     core->interfaceUserData = userData;
     core->isSuspended = 0;
 
-    char *forceGlesAllocation = getenv("SRM_FORCE_GL_ALLOCATION");
+    const char *env = getenv("SRM_DISABLE_CUSTOM_SCANOUT");
 
-    if (forceGlesAllocation && atoi(forceGlesAllocation) == 1)
-        core->forceGlesCPUBufferAllocation = 1;
+    core->customBufferScanoutIsDisabled = env && atoi(env) == 1;
+
+    SRMDebug("[core] Custom scanout enabled: %s.",
+        core->customBufferScanoutIsDisabled ? "NO" : "YES");
 
     // REF -
     if (!srmCoreUpdateEGLExtensions(core))
