@@ -1,4 +1,3 @@
-#include <private/modes/SRMRenderModeCPU.h>
 #include <private/SRMConnectorPrivate.h>
 #include <private/SRMConnectorModePrivate.h>
 #include <private/SRMDevicePrivate.h>
@@ -6,10 +5,13 @@
 #include <private/SRMCrtcPrivate.h>
 #include <private/SRMPlanePrivate.h>
 #include <private/SRMCorePrivate.h>
+#include <private/SRMBufferPrivate.h>
 
 #include <private/modes/SRMRenderModeCommon.h>
 #include <private/modes/SRMRenderModeItself.h>
+#include <private/modes/SRMRenderModePrime.h>
 #include <private/modes/SRMRenderModeDumb.h>
+#include <private/modes/SRMRenderModeCPU.h>
 
 #include <SRMLog.h>
 #include <SRMList.h>
@@ -31,15 +33,15 @@ SRMConnector *srmConnectorCreate(SRMDevice *device, UInt32 id)
     connector->device = device;
     connector->state = SRM_CONNECTOR_STATE_UNINITIALIZED;
     connector->pendingVSync = 1;
+
     pthread_mutex_init(&connector->stateMutex, NULL);
     pthread_mutex_init(&connector->propsMutex, NULL);
+
     srmConnectorUpdateProperties(connector);
-
-    // srmConnectorUpdateNames(connector) is called after its device is added to the core devices list
-
+    srmConnectorUpdateNames(connector);
     srmConnectorUpdateEncoders(connector);
     srmConnectorUpdateModes(connector);
-
+    srmConnectorSetContentType(connector, SRM_CONNECTOR_CONTENT_TYPE_GRAPHICS);
     return connector;
 }
 
@@ -59,34 +61,6 @@ void srmConnectorDestroy(SRMConnector *connector)
     free(connector);
 }
 
-// Recursively find a free id to add at the end of the name (e.g HDMI-A-0)
-// so that connectors of the same type have unique names
-UInt32 srmConnectorGetFreeNameID(SRMConnector *connector, UInt32 id)
-{
-    SRMListForeach (item1, connector->device->core->devices)
-    {
-        SRMDevice *device = srmListItemGetData(item1);
-
-        SRMListForeach (item2, device->connectors)
-        {
-            SRMConnector *otherConnector = srmListItemGetData(item2);
-
-            if (otherConnector == connector)
-                continue;
-
-            // Ups ID already used by another connector, try with the next value
-            if (otherConnector->type == connector->type && otherConnector->nameID == id)
-            {
-                id++;
-                return srmConnectorGetFreeNameID(connector, id);
-            }
-        }
-    }
-
-    // Got a free ID
-    return id;
-}
-
 UInt8 srmConnectorUpdateProperties(SRMConnector *connector)
 {
     drmModeConnector *connectorRes = drmModeGetConnector(connector->device->fd, connector->id);
@@ -102,6 +76,7 @@ UInt8 srmConnectorUpdateProperties(SRMConnector *connector)
     connector->mmWidth = connectorRes->mmWidth;
     connector->connected = connectorRes->connection == DRM_MODE_CONNECTED;
     connector->type = connectorRes->connector_type;
+    connector->nameID = connectorRes->connector_type_id;
 
     drmModeFreeConnector(connectorRes);
 
@@ -137,6 +112,8 @@ UInt8 srmConnectorUpdateProperties(SRMConnector *connector)
             connector->propIDs.link_status = prop->prop_id;
         else if (strcmp(prop->name, "non-desktop") == 0)
             connector->propIDs.non_desktop = prop->prop_id;
+        else if (strcmp(prop->name, "content type") == 0)
+            connector->propIDs.content_type = prop->prop_id;
         else if (strcmp(prop->name, "panel orientation") == 0)
             connector->propIDs.panel_orientation = prop->prop_id;
         else if (strcmp(prop->name, "subconnector") == 0)
@@ -156,9 +133,6 @@ UInt8 srmConnectorUpdateNames(SRMConnector *connector)
 {
     srmConnectorDestroyNames(connector);
 
-    // Search a free name ID for the connector type
-    connector->nameID = srmConnectorGetFreeNameID(connector, 0);
-
     // Set name
     char name[64];
     snprintf(name, sizeof(name) - 1, "%s-%d", srmGetConnectorTypeString(connector->type), connector->nameID);
@@ -470,20 +444,22 @@ void *srmConnectorRenderThread(void *conn)
 
     SRM_RENDER_MODE renderMode = srmDeviceGetRenderMode(connector->device);
 
-    SRMDebug("Connector %d device %s render mode = %s.",
-             connector->id,
+    SRMDebug("[%s] Connector %d render mode = %s.",
              connector->device->name,
+             connector->id,
              srmGetRenderModeString(renderMode));
 
     if (renderMode == SRM_RENDER_MODE_ITSELF)
         srmRenderModeItselfSetInterface(connector);
+    else if (renderMode == SRM_RENDER_MODE_PRIME)
+        srmRenderModePrimeSetInterface(connector);
     else if (renderMode == SRM_RENDER_MODE_DUMB)
         srmRenderModeDumbSetInterface(connector);
     else if (renderMode == SRM_RENDER_MODE_CPU)
         srmRenderModeCPUSetInterface(connector);
     else
     {
-        SRMError("Invalid render mode %s connector %d.",
+        SRMError("[%s] Invalid render mode for connector %d.",
                  connector->device->name,
                  connector->id);
         goto finish;
@@ -491,7 +467,7 @@ void *srmConnectorRenderThread(void *conn)
 
     if (!connector->renderInterface.initialize(connector))
     {
-        SRMError("Render mode interface initialize() failed %s connector %d.",
+        SRMError("[%s] Render mode interface initialize() failed for connector %d.",
                  connector->device->name,
                  connector->id);
         goto finish;
@@ -514,8 +490,29 @@ void *srmConnectorRenderThread(void *conn)
             if (connector->repaintRequested)
             {
                     connector->repaintRequested = 0;
+
+                    connector->inPaintGL = 1;
                     connector->renderInterface.render(connector);
-                    connector->renderInterface.flipPage(connector);
+                    connector->inPaintGL = 0;
+
+                    /* Scanning out a custom user buffer (skip the render mode interface) */
+                    if (connector->userScanoutBufferRef[0])
+                    {
+                        srmRenderModeCommonPageFlip(connector, connector->userScanoutBufferRef[0]->scanout.fb);
+                        connector->interface->pageFlipped(connector, connector->interfaceData);
+                    }
+                    /* Normal connector rendering */
+                    else
+                        connector->renderInterface.flipPage(connector);
+
+                    /* Release previous custom scanout buffer if any */
+                    if (connector->userScanoutBufferRef[0] || connector->userScanoutBufferRef[1])
+                    {
+                        srmConnectorReleaseUserScanoutBuffer(connector, 1);
+                        connector->userScanoutBufferRef[1] = connector->userScanoutBufferRef[0];
+                        connector->userScanoutBufferRef[0] = NULL;
+                    }
+
                     pthread_mutex_unlock(&connector->stateMutex);
                     continue;
             }
@@ -610,7 +607,8 @@ void srmConnectorSetCursorPlaneToNeededConnector(SRMPlane *cursorPlane)
 void srmConnectorRenderThreadCleanUp(SRMConnector *connector)
 {
     connector->renderThread = 0;
-
+    srmConnectorReleaseUserScanoutBuffer(connector, 0);
+    srmConnectorReleaseUserScanoutBuffer(connector, 1);
     srmRenderModeCommonDestroyCursor(connector);
 
     if (connector->hasRepaintCond)
@@ -678,3 +676,78 @@ void srmConnectorRenderThreadCleanUp(SRMConnector *connector)
         connector->currentModeBlobId = 0;
     }
 }
+
+void srmConnectorInitGamma(SRMConnector *connector)
+{
+    if (!connector->currentCrtc)
+        return;
+
+    UInt64 gammaSize = srmCrtcGetGammaSize(connector->currentCrtc);
+
+    if (gammaSize > 0)
+    {
+        SRMDebug("[%s] Connector (%d) gamma size = %d.",
+                 connector->device->name,
+                 connector->id,
+                 (UInt32)gammaSize);
+
+        connector->gamma = malloc(gammaSize * sizeof(*connector->gamma));
+
+        /* Apply linear gamma */
+
+        Float64 n = gammaSize - 1.0;
+        Float64 val;
+
+        if (connector->device->clientCapAtomic)
+        {
+            struct drm_color_lut *lut = connector->gamma;
+
+            for (UInt32 i = 0; i < gammaSize; i++)
+            {
+                val = (Float64)i / n;
+                lut->red = lut->green = lut->blue = (UInt16)(UINT16_MAX * val);
+                lut++;
+            }
+
+            connector->atomicChanges |= SRM_ATOMIC_CHANGE_GAMMA_LUT;
+        }
+        else
+        {
+            UInt16 *r = (UInt16*)connector->gamma;
+            UInt16 *g = r + gammaSize;
+            UInt16 *b = g + gammaSize;
+
+            for (UInt32 i = 0; i < gammaSize; i++)
+            {
+                val = (Float64)i / n;
+                r[i] = g[i] = b[i] = (UInt16)(UINT16_MAX * val);
+            }
+
+            if (drmModeCrtcSetGamma(connector->device->fd,
+                                    connector->currentCrtc->id,
+                                    (UInt32)gammaSize,
+                                    r,
+                                    g,
+                                    b))
+            {
+                SRMError("Failed to set gamma for connector %d using legacy API drmModeCrtcSetGamma().",
+                         connector->id);
+            }
+        }
+    }
+    else
+    {
+        SRMDebug("[%s] Connector (%d) does not support gamma correction.",
+                 connector->device->name,
+                 connector->id);
+    }
+}
+
+void srmConnectorReleaseUserScanoutBuffer(SRMConnector *connector, Int8 index)
+{
+    if (connector->userScanoutBufferRef[index])
+    {
+        srmBufferDestroy(connector->userScanoutBufferRef[index]);
+        connector->userScanoutBufferRef[index] = NULL;
+    }
+}
