@@ -22,6 +22,7 @@ struct SRMConnectorRenderInterface
     UInt32(*getCurrentBufferIndex)(SRMConnector *connector);
     UInt32(*getBuffersCount)(SRMConnector *connector);
     SRMBuffer*(*getBuffer)(SRMConnector *connector, UInt32 bufferIndex);
+    EGLContext (*getEGLContext)(SRMConnector *connector);
     void (*uninitialize)(SRMConnector *connector);
     void (*pause)(SRMConnector *connector);
     void (*resume)(SRMConnector *connector);
@@ -36,6 +37,7 @@ struct SRMConnectorPropIDs
     PATH,
     link_status,
     non_desktop,
+    content_type,
     panel_orientation,
     subconnector,
     vrr_capable;
@@ -53,6 +55,7 @@ struct SRMConnectorStruct
     SRMList *encoders, *modes;
     UInt32 mmWidth, mmHeight;
     SRM_CONNECTOR_SUBPIXEL subpixel;
+    SRM_CONNECTOR_CONTENT_TYPE contentType;
     SRMConnectorMode *preferredMode, *currentMode,
     *targetMode; // This one is used while changing mode
     UInt32 currentModeBlobId;
@@ -62,7 +65,6 @@ struct SRMConnectorStruct
     SRMPlane *currentPrimaryPlane, *currentCursorPlane;
     SRM_CONNECTOR_STATE state;
 
-    // Gamma table, used only in atomic API
     struct drm_color_lut *gamma;
     UInt32 gammaBlobId;
 
@@ -71,8 +73,13 @@ struct SRMConnectorStruct
     char *name, *manufacturer, *model, *serial;
 
     // Cursor
-    struct gbm_bo *cursorBO, *cursorBOPending;
-    UInt32 cursorFB, cursorFBPending;
+    struct AtomicCursor
+    {
+        struct gbm_bo *bo;
+        UInt32 fb;
+    } cursor[2];
+
+    Int32 cursorIndex;
     Int32 cursorX, cursorY;
     UInt8 cursorVisible;
     UInt32 atomicChanges;
@@ -114,8 +121,13 @@ struct SRMConnectorStruct
     pthread_mutex_t propsMutex;
 
     // Render specific
+    UInt8 allowModifiers;
     struct SRMConnectorRenderInterface renderInterface;
     void *renderData;
+    UInt8 inPaintGL;
+
+    // User scanout buffer (pending and current)
+    SRMBuffer *userScanoutBufferRef[2];
 };
 
 SRMConnector *srmConnectorCreate(SRMDevice *device, UInt32 id);
@@ -133,12 +145,14 @@ UInt8 srmConnectorUpdateModes(SRMConnector *connector);
 void srmConnectorDestroyModes(SRMConnector *connector);
 
 void srmConnectorSetCursorPlaneToNeededConnector(SRMPlane *cursorPlane);
+void srmConnectorInitGamma(SRMConnector *connector);
 
 SRMConnectorMode *srmConnectorFindPreferredMode(SRMConnector *connector);
 UInt8 srmConnectorGetBestConfiguration(SRMConnector *connector, SRMEncoder **bestEncoder, SRMCrtc **bestCrtc, SRMPlane **bestPrimaryPlane, SRMPlane **bestCursorPlane);
 void *srmConnectorRenderThread(void *conn);
 void srmConnectorUnlockRenderThread(SRMConnector *connector, UInt8 repaint);
 void srmConnectorRenderThreadCleanUp(SRMConnector *connector);
+void srmConnectorReleaseUserScanoutBuffer(SRMConnector *connector, Int8 index);
 
 #ifdef __cplusplus
 }
