@@ -89,12 +89,11 @@ struct RenderModeDataStruct
     GLuint *connectorTextures;
     UInt8 **cpuBuffers;
 
-    // Gles
     GLuint
         invertYUniform,
-        texSizeUniform,             // Texture size (width,height)
-        srcRectUniform,             // Src tex rect (x,y,width,height)
-        activeTextureUniform;       // glActiveTexture
+        texSizeUniform,
+        srcRectUniform,
+        activeTextureUniform;
 
     GLuint vertexShader,fragmentShader;
     GLuint programObject;
@@ -125,6 +124,8 @@ static UInt8 createData(SRMConnector *connector)
     data->connectorEGLSurface = EGL_NO_SURFACE;
     data->rendererEGLContext = EGL_NO_CONTEXT;
     data->rendererEGLSurface = EGL_NO_SURFACE;
+
+    srmRenderModeCommonSearchNonLinearModifier(connector);
     return 1;
 }
 
@@ -147,12 +148,7 @@ static UInt8 createGBMSurfaces(SRMConnector *connector)
         return 1;
     }
 
-    data->connectorGBMSurface = gbm_surface_create(
-        connector->device->gbm,
-        connector->currentMode->info.hdisplay,
-        connector->currentMode->info.vdisplay,
-        GBM_FORMAT_XRGB8888,
-        GBM_BO_USE_SCANOUT | GBM_BO_USE_RENDERING);
+    srmRenderModeCommonCreateConnectorGBMSurface(connector, &data->connectorGBMSurface);
 
     if (!data->connectorGBMSurface)
     {
@@ -166,7 +162,7 @@ static UInt8 createGBMSurfaces(SRMConnector *connector)
         connector->currentMode->info.hdisplay,
         connector->currentMode->info.vdisplay,
         GBM_FORMAT_XRGB8888,
-        GBM_BO_USE_RENDERING);
+        GBM_BO_USE_LINEAR | GBM_BO_USE_RENDERING);
 
     if (!data->rendererGBMSurface)
     {
@@ -175,7 +171,7 @@ static UInt8 createGBMSurfaces(SRMConnector *connector)
             connector->currentMode->info.hdisplay,
             connector->currentMode->info.vdisplay,
             GBM_FORMAT_XRGB8888,
-            GBM_BO_USE_LINEAR | GBM_BO_USE_RENDERING);
+            GBM_BO_USE_RENDERING);
 
         if (!data->rendererGBMSurface)
         {
@@ -245,46 +241,36 @@ static UInt8 createEGLContext(SRMConnector *connector)
     }
 
     if (connector->device->eglExtensions.IMG_context_priority)
-        connector->device->eglSharedContextAttribs[3] = EGL_CONTEXT_PRIORITY_MEDIUM_IMG;
+        connector->device->eglSharedContextAttribs[3] = EGL_CONTEXT_PRIORITY_HIGH_IMG;
     data->connectorEGLContext = eglCreateContext(connector->device->eglDisplay,
                                                  data->connectorEGLConfig,
-                                                 // It is EGL_NO_CONTEXT if no context was previously created in this device
                                                  connector->device->eglSharedContext,
                                                  connector->device->eglSharedContextAttribs);
 
     if (data->connectorEGLContext == EGL_NO_CONTEXT)
     {
-        SRMError("Failed to create EGL context for device %s connector %d (ITSELF MODE).",
+        SRMError("Failed to create EGL context for device %s connector %d (CPU MODE).",
                  connector->device->name,
                  connector->id);
         return 0;
     }
 
-    // Store it if is the first context created for this device, so that later on shared context can be created
-    if (connector->device->eglSharedContext == EGL_NO_CONTEXT)
-        connector->device->eglSharedContext = data->connectorEGLContext;
-
     if (connector->device->rendererDevice->eglExtensions.IMG_context_priority)
         connector->device->rendererDevice->eglSharedContextAttribs[3] = EGL_CONTEXT_PRIORITY_HIGH_IMG;
 
     data->rendererEGLContext = eglCreateContext(connector->device->rendererDevice->eglDisplay,
-                                                 data->rendererEGLConfig,
-                                                 // It is EGL_NO_CONTEXT if no context was previously created in this device
-                                                 connector->device->rendererDevice->eglSharedContext,
+                                                data->rendererEGLConfig,
+                                                connector->device->rendererDevice->eglSharedContext,
                                                 connector->device->rendererDevice->eglSharedContextAttribs);
 
     if (data->rendererEGLContext == EGL_NO_CONTEXT)
     {
-        SRMError("Failed to create EGL context for device %s connector %d (ITSELF MODE).",
+        SRMError("Failed to create EGL context for device %s connector %d (CPU MODE).",
                  connector->device->name,
                  connector->id);
         return 0;
     }
 
-    // Store it if is the first context created for this device, so that later on shared context can be created
-    if (connector->device->rendererDevice->eglSharedContext == EGL_NO_CONTEXT)
-        connector->device->rendererDevice->eglSharedContext = data->rendererEGLContext;
-
     return 1;
 }
 
@@ -294,17 +280,13 @@ static void destroyEGLContext(SRMConnector *connector)
 
     if (data->connectorEGLContext != EGL_NO_CONTEXT)
     {
-        if (data->connectorEGLContext != connector->device->eglSharedContext)
-            eglDestroyContext(connector->device->eglDisplay, data->connectorEGLContext);
-
+        eglDestroyContext(connector->device->eglDisplay, data->connectorEGLContext);
         data->connectorEGLContext = EGL_NO_CONTEXT;
     }
 
     if (data->rendererEGLContext != EGL_NO_CONTEXT)
     {
-        if (data->rendererEGLContext != connector->device->rendererDevice->eglSharedContext)
-            eglDestroyContext(connector->device->rendererDevice->eglDisplay, data->rendererEGLContext);
-
+        eglDestroyContext(connector->device->rendererDevice->eglDisplay, data->rendererEGLContext);
         data->rendererEGLContext = EGL_NO_CONTEXT;
     }
 }
@@ -341,13 +323,28 @@ static UInt8 createEGLSurfaces(SRMConnector *connector)
 
     struct gbm_bo *bo = NULL;
 
-    while (srmListGetLength(bos) < 1 && gbm_surface_has_free_buffers(data->connectorGBMSurface) > 0)
+    char *env = getenv("SRM_RENDER_MODE_CPU_FB_COUNT");
+
+    UInt32 fbCount = 2;
+
+    if (env)
+    {
+        Int32 c = atoi(env);
+
+        if (c > 1 && c < 4)
+            fbCount = c;
+    }
+
+    do
     {
         eglSwapBuffers(connector->device->eglDisplay,
                        data->connectorEGLSurface);
         bo = gbm_surface_lock_front_buffer(data->connectorGBMSurface);
-        srmListAppendData(bos, bo);
+
+        if (bo)
+            srmListAppendData(bos, bo);
     }
+    while (srmListGetLength(bos) < fbCount && gbm_surface_has_free_buffers(data->connectorGBMSurface) > 0 && bo);
 
     data->buffersCount = srmListGetLength(bos);
 
@@ -394,13 +391,16 @@ static UInt8 createEGLSurfaces(SRMConnector *connector)
     bos = srmListCreate();
     bo = NULL;
 
-    while (srmListGetLength(bos) < 1 && gbm_surface_has_free_buffers(data->rendererGBMSurface) > 0)
+    do
     {
         eglSwapBuffers(connector->device->rendererDevice->eglDisplay,
                        data->rendererEGLSurface);
         bo = gbm_surface_lock_front_buffer(data->rendererGBMSurface);
-        srmListAppendData(bos, bo);
+
+        if (bo)
+            srmListAppendData(bos, bo);
     }
+    while (srmListGetLength(bos) < fbCount && gbm_surface_has_free_buffers(data->rendererGBMSurface) > 0 && bo);
 
     data->buffersCount = srmListGetLength(bos);
 
@@ -583,6 +583,7 @@ static UInt8 createGLES2(SRMConnector *connector)
     data->activeTextureUniform = glGetUniformLocation(data->programObject, "tex");
 
     data->cpuBuffers = calloc(data->buffersCount, sizeof(UInt8*));
+
     data->connectorTextures = calloc(data->buffersCount, sizeof(GLuint));
 
     for (UInt32 i = 0; i < data->buffersCount; i++)
@@ -688,6 +689,35 @@ static UInt8 createDRMFramebuffers(SRMConnector *connector)
 
     for (UInt32 i = 0; i < data->buffersCount; i++)
     {
+        if (connector->currentFormat.modifier != DRM_FORMAT_MOD_LINEAR)
+        {
+            UInt32 boHandles[4] = { 0 };
+            boHandles[0] = gbm_bo_get_handle_for_plane(data->connectorBOs[i], 0).u32;
+
+            UInt32 pitches[4] = { 0 };
+            pitches[0] = gbm_bo_get_stride_for_plane(data->connectorBOs[i], 0);
+
+            UInt32 offsets[4] = { 0 };
+            offsets[0] = gbm_bo_get_offset(data->connectorBOs[i], 0);
+
+            UInt64 mods[4] = { 0 };
+            mods[0] = connector->currentFormat.modifier;
+
+            ret = drmModeAddFB2WithModifiers(connector->device->fd,
+                                             connector->currentMode->info.hdisplay,
+                                             connector->currentMode->info.vdisplay,
+                                             connector->currentFormat.format,
+                                             boHandles,
+                                             pitches,
+                                             offsets,
+                                             mods,
+                                             &data->connectorDRMFramebuffers[i],
+                                             DRM_MODE_FB_MODIFIERS);
+
+            if (ret == 0)
+                goto skipLegacy;
+        }
+
         ret = drmModeAddFB(connector->device->fd,
                            connector->currentMode->info.hdisplay,
                            connector->currentMode->info.vdisplay,
@@ -706,6 +736,8 @@ static UInt8 createDRMFramebuffers(SRMConnector *connector)
             return 0;
         }
 
+        skipLegacy:
+
         data->rendererBuffers[i] = srmBufferCreateFromGBM(connector->device->rendererDevice->core, data->rendererBOs[i]);
 
         if (!data->rendererBuffers[i])
@@ -887,7 +919,7 @@ static UInt8 flipPage(SRMConnector *connector)
     }
     else
     {
-        glFlush();
+        glFinish();
     }
 
     swapBuffers(connector, connector->device->rendererDevice->eglDisplay, data->rendererEGLSurface);
@@ -1007,9 +1039,13 @@ static void uninitialize(SRMConnector *connector)
 
 static UInt8 initialize(SRMConnector *connector)
 {
+    connector->allowModifiers = 1;
+
+retry:
+
     if (!eglBindAPI(EGL_OPENGL_ES_API))
     {
-        SRMError("Failed to bind GLES API for device %s connector %d (ITSELF MODE).",
+        SRMError("Failed to bind GLES API for device %s connector %d (CPU MODE).",
                  connector->device->name,
                  connector->id);
         goto fail;
@@ -1042,11 +1078,20 @@ static UInt8 initialize(SRMConnector *connector)
     return 1;
 
 fail:
-    SRMError("Failed to initialize render mode ITSELF for device %s connector %d.",
+    uninitialize(connector);
+
+    if (connector->allowModifiers)
+    {
+        SRMError("Failed to initialize device %s connector %d with explicit modifiers, falling back to implicit modifiers (CPU MODE).",
+                 connector->device->name,
+                 connector->id);
+        connector->allowModifiers = 0;
+        goto retry;
+    }
+
+    SRMError("Failed to initialize render mode CPU for device %s connector %d.",
              connector->device->name,
              connector->id);
-
-    uninitialize(connector);
     return 0;
 }
 
