@@ -4,6 +4,7 @@
 #include <private/SRMDevicePrivate.h>
 #include <private/SRMConnectorPrivate.h>
 #include <private/SRMConnectorModePrivate.h>
+#include <SRMList.h>
 
 #include <SRMCore.h>
 #include <SRMLog.h>
@@ -147,90 +148,54 @@ UInt8 srmRenderModeCommonCreateCursor(SRMConnector *connector)
             return 0;
     }
 
-    connector->cursorBO = gbm_bo_create(connector->device->gbm,
-                                        64,
-                                        64,
-                                        GBM_FORMAT_ARGB8888,
-                                        GBM_BO_USE_CURSOR | GBM_BO_USE_WRITE);
+    Int32 ret;
 
-    if (connector->cursorBO)
+    connector->cursorIndex = 0;
+
+    for (int i = 0; i < 2; i++)
     {
-        if (connector->device->clientCapAtomic)
-        {
-            Int32 ret = drmModeAddFB(connector->device->fd,
-                         gbm_bo_get_width(connector->cursorBO),
-                         gbm_bo_get_height(connector->cursorBO),
-                         32,
-                         gbm_bo_get_bpp(connector->cursorBO),
-                         gbm_bo_get_stride(connector->cursorBO),
-                         gbm_bo_get_handle(connector->cursorBO).u32,
-                         &connector->cursorFB);
+        connector->cursor[i].bo = gbm_bo_create(connector->device->gbm,
+                                                      64,
+                                                      64,
+                                                      GBM_FORMAT_ARGB8888,
+                                                      GBM_BO_USE_CURSOR | GBM_BO_USE_WRITE);
 
-            if (ret)
-            {
-                SRMError("Failed to setup hw cursor for connector %d.", connector->id);
-                goto fail;
-            }
-        }
+        if (!connector->cursor[i].bo)
+            goto fail;
+
+        if (!connector->device->clientCapAtomic)
+            continue;
+
+        ret = drmModeAddFB(connector->device->fd,
+                                 gbm_bo_get_width(connector->cursor[i].bo),
+                                 gbm_bo_get_height(connector->cursor[i].bo),
+                                 32,
+                                 gbm_bo_get_bpp(connector->cursor[i].bo),
+                                 gbm_bo_get_stride(connector->cursor[i].bo),
+                                 gbm_bo_get_handle(connector->cursor[i].bo).u32,
+                                 &connector->cursor[i].fb);
+
+        if (ret)
+            goto fail;
     }
-    else
-        return 0;
 
-    connector->cursorBOPending = gbm_bo_create(connector->device->gbm,
-                                        64,
-                                        64,
-                                        GBM_FORMAT_ARGB8888,
-                                        GBM_BO_USE_CURSOR | GBM_BO_USE_WRITE);
-
-    if (connector->cursorBOPending)
-    {
-        if (connector->device->clientCapAtomic)
-        {
-            Int32 ret = drmModeAddFB(connector->device->fd,
-                         gbm_bo_get_width(connector->cursorBOPending),
-                         gbm_bo_get_height(connector->cursorBOPending),
-                         32,
-                         gbm_bo_get_bpp(connector->cursorBOPending),
-                         gbm_bo_get_stride(connector->cursorBOPending),
-                         gbm_bo_get_handle(connector->cursorBOPending).u32,
-                         &connector->cursorFBPending);
-
-            if (ret)
-            {
-                SRMError("Failed to setup hw cursor for connector %d.", connector->id);
-                goto fail;
-            }
-
-            return 1;
-        }
-
-        return 1;
-    }
+    return 1;
 
     fail:
 
-    if (connector->cursorFB)
+    for (int i = 0; i < 2; i++)
     {
-        drmModeRmFB(connector->device->fd, connector->cursorFB);
-        connector->cursorFB = 0;
-    }
+        if (connector->cursor[i].fb != 0)
+        {
+            drmModeRmFB(connector->device->fd, connector->cursor[i].fb);
+            connector->cursor[i].fb = 0;
+        }
 
-    if (connector->cursorBO)
-    {
-        gbm_bo_destroy(connector->cursorBO);
-        connector->cursorBO = NULL;
-    }
-
-    if (connector->cursorFBPending)
-    {
-        drmModeRmFB(connector->device->fd, connector->cursorFBPending);
-        connector->cursorFBPending = 0;
-    }
-
-    if (connector->cursorBOPending)
-    {
-        gbm_bo_destroy(connector->cursorBOPending);
-        connector->cursorBOPending = NULL;
+        if (connector->cursor[i].bo)
+        {
+            gbm_bo_destroy(connector->cursor[i].bo);
+            connector->cursor[i].bo = NULL;
+        }
     }
 
     SRMError("Failed to setup hw cursor for connector %d.", connector->id);
@@ -268,8 +233,6 @@ UInt8 srmRenderModeCommonWaitRepaintRequest(SRMConnector *connector)
 
 void srmRenderModeCommitAtomicChanges(SRMConnector *connector, drmModeAtomicReqPtr req, UInt8 clearFlags)
 {
-    pthread_mutex_lock(&connector->propsMutex);
-
     if (connector->currentCursorPlane)
     {
         UInt8 updatedFB = 0;
@@ -279,24 +242,20 @@ void srmRenderModeCommitAtomicChanges(SRMConnector *connector, drmModeAtomicReqP
             if (clearFlags)
                 connector->atomicChanges &= ~SRM_ATOMIC_CHANGE_CURSOR_BUFFER;
 
-            struct gbm_bo *tmpBo = connector->cursorBO;
-            connector->cursorBO = connector->cursorBOPending;
-            connector->cursorBOPending = tmpBo;
-
-            UInt32 tmpFb = connector->cursorFB;
-            connector->cursorFB = connector->cursorFBPending;
-            connector->cursorFBPending = tmpFb;
+            connector->cursorIndex = 1 - connector->cursorIndex;
 
             if (connector->cursorVisible)
             {
                 drmModeAtomicAddProperty(req,
                                         connector->currentCursorPlane->id,
                                         connector->currentCursorPlane->propIDs.FB_ID,
-                                        connector->cursorFB);
+                                        connector->cursor[connector->cursorIndex].fb);
                 updatedFB = 1;
             }
         }
 
+        UInt8 updatedVisibility = 0;
+
         if (connector->atomicChanges & SRM_ATOMIC_CHANGE_CURSOR_VISIBILITY)
         {
             if (clearFlags)
@@ -304,12 +263,14 @@ void srmRenderModeCommitAtomicChanges(SRMConnector *connector, drmModeAtomicReqP
 
             if (connector->cursorVisible)
             {
+                updatedVisibility = 1;
+
                 if (!updatedFB)
                 {
                     drmModeAtomicAddProperty(req,
                                             connector->currentCursorPlane->id,
                                             connector->currentCursorPlane->propIDs.FB_ID,
-                                            connector->cursorFB);
+                                            connector->cursor[connector->cursorIndex].fb);
                 }
 
                 drmModeAtomicAddProperty(req,
@@ -317,6 +278,16 @@ void srmRenderModeCommitAtomicChanges(SRMConnector *connector, drmModeAtomicReqP
                                         connector->currentCursorPlane->propIDs.CRTC_ID,
                                         connector->currentCrtc->id);
 
+                drmModeAtomicAddProperty(req,
+                                         connector->currentCursorPlane->id,
+                                         connector->currentCursorPlane->propIDs.CRTC_X,
+                                         connector->cursorX);
+
+                drmModeAtomicAddProperty(req,
+                                         connector->currentCursorPlane->id,
+                                         connector->currentCursorPlane->propIDs.CRTC_Y,
+                                         connector->cursorY);
+
                 drmModeAtomicAddProperty(req,
                                         connector->currentCursorPlane->id,
                                         connector->currentCursorPlane->propIDs.CRTC_W,
@@ -366,15 +337,18 @@ void srmRenderModeCommitAtomicChanges(SRMConnector *connector, drmModeAtomicReqP
             if (clearFlags)
                 connector->atomicChanges &= ~SRM_ATOMIC_CHANGE_CURSOR_POSITION;
 
-            drmModeAtomicAddProperty(req,
-                                    connector->currentCursorPlane->id,
-                                    connector->currentCursorPlane->propIDs.CRTC_X,
-                                    connector->cursorX);
+            if (!updatedVisibility)
+            {
+                drmModeAtomicAddProperty(req,
+                                        connector->currentCursorPlane->id,
+                                        connector->currentCursorPlane->propIDs.CRTC_X,
+                                        connector->cursorX);
 
-            drmModeAtomicAddProperty(req,
-                                    connector->currentCursorPlane->id,
-                                    connector->currentCursorPlane->propIDs.CRTC_Y,
-                                    connector->cursorY);
+                drmModeAtomicAddProperty(req,
+                                        connector->currentCursorPlane->id,
+                                        connector->currentCursorPlane->propIDs.CRTC_Y,
+                                        connector->cursorY);
+            }
         }
     }
 
@@ -406,7 +380,17 @@ void srmRenderModeCommitAtomicChanges(SRMConnector *connector, drmModeAtomicReqP
         }
     }
 
-    pthread_mutex_unlock(&connector->propsMutex);
+    if (connector->atomicChanges & SRM_ATOMIC_CHANGE_CONTENT_TYPE)
+    {
+        if (clearFlags)
+            connector->atomicChanges &= ~SRM_ATOMIC_CHANGE_CONTENT_TYPE;
+
+        if (connector->propIDs.content_type)
+            drmModeAtomicAddProperty(req,
+                                     connector->id,
+                                     connector->propIDs.content_type,
+                                     connector->contentType);
+    }
 }
 
 void srmRenderModeCommonDestroyCursor(SRMConnector *connector)
@@ -433,32 +417,24 @@ void srmRenderModeCommonDestroyCursor(SRMConnector *connector)
         }
     }
 
-    if (connector->cursorFB)
+    for (int i = 0; i < 2; i++)
     {
-        drmModeRmFB(connector->device->fd, connector->cursorFB);
-        connector->cursorFB = 0;
-    }
+        if (connector->cursor[i].fb != 0)
+        {
+            drmModeRmFB(connector->device->fd, connector->cursor[i].fb);
+            connector->cursor[i].fb = 0;
+        }
 
-    if (connector->cursorBO)
-    {
-        gbm_bo_destroy(connector->cursorBO);
-        connector->cursorBO = NULL;
-    }
-
-    if (connector->cursorFBPending)
-    {
-        drmModeRmFB(connector->device->fd, connector->cursorFBPending);
-        connector->cursorFBPending = 0;
-    }
-
-    if (connector->cursorBOPending)
-    {
-        gbm_bo_destroy(connector->cursorBOPending);
-        connector->cursorBOPending = NULL;
+        if (connector->cursor[i].bo)
+        {
+            gbm_bo_destroy(connector->cursor[i].bo);
+            connector->cursor[i].bo = NULL;
+        }
     }
 
     connector->atomicChanges &= ~(SRM_ATOMIC_CHANGE_CURSOR_VISIBILITY | SRM_ATOMIC_CHANGE_CURSOR_POSITION | SRM_ATOMIC_CHANGE_CURSOR_BUFFER);
     connector->cursorVisible = 0;
+    connector->cursorIndex = 0;
 }
 
 Int32 srmRenderModeAtomicCommit(Int32 fd, drmModeAtomicReqPtr req, UInt32 flags, void *data, UInt8 forceRetry)
@@ -591,7 +567,8 @@ Int32 srmRenderModeCommonUpdateMode(SRMConnector *connector, UInt32 fb)
                                      connector->currentPrimaryPlane->propIDs.SRC_H,
                                      (UInt64)connector->currentMode->info.vdisplay << 16);
 
-            srmRenderModeCommitAtomicChanges(connector, req, 1);
+            UInt32 prevCursorIndex = connector->cursorIndex;
+            srmRenderModeCommitAtomicChanges(connector, req, 0);
 
             ret = srmRenderModeAtomicCommit(connector->device->fd,
                                 req,
@@ -600,10 +577,13 @@ Int32 srmRenderModeCommonUpdateMode(SRMConnector *connector, UInt32 fb)
 
             if (ret)
             {
+                connector->cursorIndex = prevCursorIndex;
                 SRMError("Failed set mode with same size on device %s connector %d. Error: %d. (atomic)",
                          connector->device->name,
                          connector->id, ret);
             }
+            else
+                connector->atomicChanges = 0;
 
             drmModeAtomicFree(req);
         }
@@ -690,6 +670,9 @@ Int32 srmRenderModeCommonUpdateMode(SRMConnector *connector, UInt32 fb)
 
 void srmRenderModeCommonUninitialize(SRMConnector *connector)
 {
+    if (connector->state == SRM_CONNECTOR_STATE_INITIALIZING)
+        return;
+
     Int32 ret;
 
     if (connector->device->clientCapAtomic)
@@ -736,7 +719,6 @@ void srmRenderModeCommonPauseRendering(SRMConnector *connector)
             SRMWarning("Failed to reset CRTC device %s connector %d. Error: %d (not DRM master). (atomic)",
                      connector->device->name,
                      connector->id, ret);
-
         }
     }
     else
@@ -764,6 +746,8 @@ void srmRenderModeCommonResumeRendering(SRMConnector *connector, UInt32 fb)
     Int32 ret;
     connector->lastFb = fb;
 
+    srmRenderModeCommonSyncState(connector);
+
     if (connector->device->clientCapAtomic)
     {
         drmModeAtomicReqPtr req;
@@ -854,7 +838,8 @@ void srmRenderModeCommonResumeRendering(SRMConnector *connector, UInt32 fb)
                                  connector->currentPrimaryPlane->propIDs.SRC_H,
                                  (UInt64)connector->currentMode->info.vdisplay << 16);
 
-        srmRenderModeCommitAtomicChanges(connector, req, 1);
+        UInt32 prevCursorIndex = connector->cursorIndex;
+        srmRenderModeCommitAtomicChanges(connector, req, 0);
 
         // Commit
         ret = srmRenderModeAtomicCommit(connector->device->fd,
@@ -866,10 +851,13 @@ void srmRenderModeCommonResumeRendering(SRMConnector *connector, UInt32 fb)
 
         if (ret)
         {
+            connector->cursorIndex = prevCursorIndex;
             SRMError("Failed to resume crtc mode on device %s connector %d.",
                      connector->device->name,
                      connector->id);
         }
+        else
+            connector->atomicChanges = 0;
     }
     else
     {
@@ -986,7 +974,8 @@ Int32 srmRenderModeCommonInitCrtc(SRMConnector *connector, UInt32 fb)
                                  connector->currentPrimaryPlane->propIDs.SRC_H,
                                  (UInt64)connector->currentMode->info.vdisplay << 16);
 
-        srmRenderModeCommitAtomicChanges(connector, req, 1);
+        UInt32 prevCursorIndex = connector->cursorIndex;
+        srmRenderModeCommitAtomicChanges(connector, req, 0);
 
         // Commit
         ret = srmRenderModeAtomicCommit(connector->device->fd,
@@ -998,12 +987,16 @@ Int32 srmRenderModeCommonInitCrtc(SRMConnector *connector, UInt32 fb)
 
         if (ret)
         {
+            connector->cursorIndex = prevCursorIndex;
             SRMError("Failed to set crtc mode on device %s connector %d (atomic).",
                     connector->device->name,
                     connector->id);
         }
         else
+        {
+            connector->atomicChanges = 0;
             goto skipLegacy;
+        }
     }
 
     /* Occasionally, the Atomic API fails to set the connector CRTC for reasons unknown.
@@ -1049,18 +1042,23 @@ void srmRenderModeCommonPageFlip(SRMConnector *connector, UInt32 fb)
     Int32 ret = 0;
 
     UInt32 buffersCount = srmConnectorGetBuffersCount(connector);
+    UInt8 customScanoutBuffer = connector->userScanoutBufferRef[0] != NULL;
 
-    if (connector->pendingPageFlip || buffersCount == 1 || buffersCount > 2)
+    if (customScanoutBuffer || connector->pendingPageFlip || buffersCount == 1 || buffersCount > 2)
         srmRenderModeCommonWaitPageFlip(connector, -1);
 
     connector->lastFb = fb;
 
     if (connector->device->clientCapAtomic)
     {
+        pthread_mutex_lock(&connector->propsMutex);
+
         if (connector->currentVSync)
         {
             drmModeAtomicReqPtr req;
             req = drmModeAtomicAlloc();
+
+            UInt32 prevCursorIndex = connector->cursorIndex;
             srmRenderModeCommitAtomicChanges(connector, req, 0);
             drmModeAtomicAddProperty(req,
                                      connector->currentPrimaryPlane->id,
@@ -1069,8 +1067,10 @@ void srmRenderModeCommonPageFlip(SRMConnector *connector, UInt32 fb)
             ret = srmRenderModeAtomicCommit(connector->device->fd,
                                         req,
                                         DRM_MODE_PAGE_FLIP_EVENT | DRM_MODE_ATOMIC_NONBLOCK,
-                                        connector, 1);
-            if (ret == 0 && !connector->firstPageFlip)
+                                        connector, 0);
+            if (ret)
+                connector->cursorIndex = prevCursorIndex;
+            else
                 connector->atomicChanges = 0;
 
             drmModeAtomicFree(req);
@@ -1089,7 +1089,7 @@ void srmRenderModeCommonPageFlip(SRMConnector *connector, UInt32 fb)
                 ret = srmRenderModeAtomicCommit(connector->device->fd,
                                                 req,
                                                 DRM_MODE_PAGE_FLIP_EVENT | DRM_MODE_PAGE_FLIP_ASYNC | DRM_MODE_ATOMIC_NONBLOCK,
-                                                connector, 1);
+                                                connector, 0);
                 drmModeAtomicFree(req);
             }
 
@@ -1097,6 +1097,7 @@ void srmRenderModeCommonPageFlip(SRMConnector *connector, UInt32 fb)
             {
                 drmModeAtomicReqPtr req;
                 req = drmModeAtomicAlloc();
+                UInt32 prevCursorIndex = connector->cursorIndex;
                 srmRenderModeCommitAtomicChanges(connector, req, 0);
                 drmModeAtomicAddProperty(req,
                                          connector->currentPrimaryPlane->id,
@@ -1105,16 +1106,19 @@ void srmRenderModeCommonPageFlip(SRMConnector *connector, UInt32 fb)
                 ret = srmRenderModeAtomicCommit(connector->device->fd,
                                                 req,
                                                 DRM_MODE_PAGE_FLIP_EVENT | DRM_MODE_ATOMIC_NONBLOCK,
-                                                connector, 1);
+                                                connector, 0);
                 drmModeAtomicFree(req);
 
-                // Clear flags on success
-                if (ret == 0 && !connector->firstPageFlip)
+                if (ret)
+                    connector->cursorIndex = prevCursorIndex;
+                else
                     connector->atomicChanges = 0;
             }
 
             connector->pendingPageFlip = 1;
         }
+
+        pthread_mutex_unlock(&connector->propsMutex);
     }
     else
     {
@@ -1154,9 +1158,21 @@ void srmRenderModeCommonPageFlip(SRMConnector *connector, UInt32 fb)
         SRMError("Failed to page flip on device %s connector %d. Error: %d.",
                  connector->device->name,
                  connector->id, ret);
+
+        if (customScanoutBuffer && ret == -22)
+        {
+            if (!srmFormatIsInList(
+                    connector->currentPrimaryPlane->inFormatsBlacklist,
+                    connector->userScanoutBufferRef[0]->scanout.fmt.format,
+                    connector->userScanoutBufferRef[0]->scanout.fmt.modifier))
+                srmFormatsListAddFormat(
+                    connector->currentPrimaryPlane->inFormatsBlacklist,
+                    connector->userScanoutBufferRef[0]->scanout.fmt.format,
+                    connector->userScanoutBufferRef[0]->scanout.fmt.modifier);
+        }
     }
 
-    if (buffersCount == 2 || connector->firstPageFlip)
+    if (customScanoutBuffer || buffersCount == 2 || connector->firstPageFlip)
     {
         connector->firstPageFlip = 0;
         srmRenderModeCommonWaitPageFlip(connector, -1);
@@ -1284,7 +1300,7 @@ Int32 srmRenderModeAtomicResetConnectorProps(SRMConnector *connector)
                              connector->currentPrimaryPlane->propIDs.SRC_H,
                              0);
 
-    srmRenderModeCommitAtomicChanges(connector, req, 1);
+    srmRenderModeCommitAtomicChanges(connector, req, 0);
 
     ret = srmRenderModeAtomicCommit(connector->device->fd,
                                     req,
@@ -1295,3 +1311,123 @@ Int32 srmRenderModeAtomicResetConnectorProps(SRMConnector *connector)
 
     return ret;
 }
+
+/* This ensures properties are restored after resuming */
+void srmRenderModeCommonSyncState(SRMConnector *connector)
+{
+    if (!connector->currentCrtc)
+        return;
+
+    if (connector->device->clientCapAtomic)
+    {
+        if (connector->propIDs.content_type)
+            connector->atomicChanges |= SRM_ATOMIC_CHANGE_CONTENT_TYPE;
+
+        if (connector->cursor[0].bo)
+            connector->atomicChanges |= SRM_ATOMIC_CHANGE_CURSOR_POSITION | SRM_ATOMIC_CHANGE_CURSOR_VISIBILITY;
+
+        if (connector->gamma)
+            connector->atomicChanges |= SRM_ATOMIC_CHANGE_GAMMA_LUT;
+    }
+    else
+    {
+        if (connector->propIDs.content_type)
+            drmModeConnectorSetProperty(connector->device->fd,
+                                        connector->id,
+                                        connector->propIDs.content_type,
+                                        connector->contentType);
+
+        if (connector->cursor[0].bo)
+        {
+            if (connector->cursorVisible)
+                drmModeSetCursor(connector->device->fd,
+                                 connector->currentCrtc->id,
+                                 gbm_bo_get_handle(connector->cursor[connector->cursorIndex].bo).u32,
+                                 64,
+                                 64);
+            else
+                drmModeSetCursor(connector->device->fd,
+                                 connector->currentCrtc->id,
+                                 0,
+                                 0,
+                                 0);
+
+            drmModeMoveCursor(connector->device->fd,
+                              connector->currentCrtc->id,
+                              connector->cursorX,
+                              connector->cursorY);
+        }
+
+        /* This is always != NULL if gammaSize > 0 */
+        if (connector->gamma)
+        {
+            UInt16 *table = (UInt16*)connector->gamma;
+            UInt64 gammaSize = srmConnectorGetGammaSize(connector);
+            if (drmModeCrtcSetGamma(connector->device->fd,
+                                    connector->currentCrtc->id,
+                                    (UInt32)gammaSize,
+                                    table,
+                                    table + gammaSize,
+                                    table + gammaSize + gammaSize))
+            {
+                SRMError("Failed to set gamma for connector %d using legacy API drmModeCrtcSetGamma().",
+                         connector->id);
+            }
+        }
+    }
+}
+
+void srmRenderModeCommonSearchNonLinearModifier(SRMConnector *connector)
+{
+    connector->currentFormat.format = DRM_FORMAT_XRGB8888;
+    connector->currentFormat.modifier = DRM_FORMAT_MOD_LINEAR;
+
+    if (!connector->device->clientCapAtomic || !connector->device->capAddFb2Modifiers || !connector->allowModifiers)
+        return;
+
+    SRMListForeach(it, connector->currentPrimaryPlane->inFormats)
+    {
+        SRMFormat *fmt = srmListItemGetData(it);
+
+        if (fmt->format == DRM_FORMAT_XRGB8888
+            && fmt->modifier != DRM_FORMAT_MOD_LINEAR
+            && fmt->modifier != DRM_FORMAT_MOD_INVALID
+            && srmFormatIsInList(srmDeviceGetDMATextureFormats(connector->device), fmt->format, fmt->modifier))
+        {
+            SRMDebug("[%s] Connector %d using format: %s - %s.",
+                connector->device->name,
+                connector->id,
+                drmGetFormatName(fmt->format),
+                drmGetFormatModifierName(fmt->modifier));
+            connector->currentFormat.modifier = fmt->modifier;
+            break;
+        }
+    }
+}
+
+void srmRenderModeCommonCreateConnectorGBMSurface(SRMConnector *connector, struct gbm_surface **surface)
+{
+    if (connector->currentFormat.modifier != DRM_FORMAT_MOD_LINEAR)
+    {
+        *surface = gbm_surface_create_with_modifiers2(
+            connector->device->gbm,
+            connector->currentMode->info.hdisplay,
+            connector->currentMode->info.vdisplay,
+            connector->currentFormat.format,
+            &connector->currentFormat.modifier,
+            1,
+            GBM_BO_USE_SCANOUT | GBM_BO_USE_RENDERING);
+
+        if (*surface)
+            return;
+
+        connector->currentFormat.modifier = DRM_FORMAT_MOD_LINEAR;
+    }
+
+    *surface = gbm_surface_create(
+        connector->device->gbm,
+        connector->currentMode->info.hdisplay,
+        connector->currentMode->info.vdisplay,
+        GBM_FORMAT_XRGB8888,
+        GBM_BO_USE_SCANOUT | GBM_BO_USE_RENDERING);
+}
