@@ -62,21 +62,7 @@ static UInt8 createData(SRMConnector *connector)
     data->connectorEGLContext = EGL_NO_CONTEXT;
     data->connectorEGLSurface = EGL_NO_SURFACE;
 
-    connector->currentFormat.format = 0;
-
-    // TODO: check if match visual ID
-    if (srmFormatListFirstMatchFormat(connector->currentPrimaryPlane->inFormats, DRM_FORMAT_XRGB8888))
-        connector->currentFormat.format = DRM_FORMAT_XRGB8888;
-    else if (srmFormatListFirstMatchFormat(connector->currentPrimaryPlane->inFormats, DRM_FORMAT_XBGR8888))
-        connector->currentFormat.format = DRM_FORMAT_XBGR8888;
-    else
-    {
-        SRMError("Failed to initialize device %s connector %d. XRGB8888 and XBGR8888 not avaliable (ITSELF MODE).",
-                 connector->device->name,
-                 connector->id);
-        return 0;
-    }
-
+    srmRenderModeCommonSearchNonLinearModifier(connector);
     return 1;
 }
 
@@ -99,12 +85,7 @@ static UInt8 createGBMSurfaces(SRMConnector *connector)
         return 1;
     }
 
-    data->connectorGBMSurface = gbm_surface_create(
-        connector->device->gbm,
-        connector->currentMode->info.hdisplay,
-        connector->currentMode->info.vdisplay,
-        connector->currentFormat.format,
-        GBM_BO_USE_SCANOUT | GBM_BO_USE_RENDERING);
+    srmRenderModeCommonCreateConnectorGBMSurface(connector, &data->connectorGBMSurface);
 
     if (!data->connectorGBMSurface)
     {
@@ -160,7 +141,6 @@ static UInt8 createEGLContext(SRMConnector *connector)
 
     data->connectorEGLContext = eglCreateContext(connector->device->eglDisplay,
                                                  data->connectorEGLConfig,
-                                                 // It is EGL_NO_CONTEXT if no context was previously created in this device
                                                  connector->device->eglSharedContext,
                                                  connector->device->eglSharedContextAttribs);
 
@@ -179,10 +159,6 @@ static UInt8 createEGLContext(SRMConnector *connector)
         SRMDebug("[%s] Using %s priority EGL context.", srmConnectorGetName(connector), priority == EGL_CONTEXT_PRIORITY_HIGH_IMG ? "high" : "medium");
     }
 
-    // Store it if is the first context created for this device, so that later on shared context can be created
-    if (connector->device->eglSharedContext == EGL_NO_CONTEXT)
-        connector->device->eglSharedContext = data->connectorEGLContext;
-
     return 1;
 }
 
@@ -192,9 +168,7 @@ static void destroyEGLContext(SRMConnector *connector)
 
     if (data->connectorEGLContext != EGL_NO_CONTEXT)
     {
-        if (data->connectorEGLContext != connector->device->eglSharedContext)
-            eglDestroyContext(connector->device->eglDisplay, data->connectorEGLContext);
-
+        eglDestroyContext(connector->device->eglDisplay, data->connectorEGLContext);
         data->connectorEGLContext = EGL_NO_CONTEXT;
     }
 }
@@ -239,17 +213,21 @@ static UInt8 createEGLSurfaces(SRMConnector *connector)
     {
         Int32 c = atoi(env);
 
-        if (c > 0 && c < 4)
+        if (c > 1 && c < 4)
             fbCount = c;
     }
 
-    while (srmListGetLength(bos) < fbCount && gbm_surface_has_free_buffers(data->connectorGBMSurface) > 0)
+    do
     {
         eglSwapBuffers(connector->device->rendererDevice->eglDisplay,
                        data->connectorEGLSurface);
+
         bo = gbm_surface_lock_front_buffer(data->connectorGBMSurface);
-        srmListAppendData(bos, bo);
+
+        if (bo)
+            srmListAppendData(bos, bo);
     }
+    while (srmListGetLength(bos) < fbCount && gbm_surface_has_free_buffers(data->connectorGBMSurface) > 0 && bo);
 
     data->buffersCount = srmListGetLength(bos);
 
@@ -325,6 +303,35 @@ static UInt8 createDRMFramebuffers(SRMConnector *connector)
 
     for (UInt32 i = 0; i < data->buffersCount; i++)
     {
+        if (connector->currentFormat.modifier != DRM_FORMAT_MOD_LINEAR)
+        {
+            UInt32 boHandles[4] = { 0 };
+            boHandles[0] = gbm_bo_get_handle_for_plane(data->connectorBOs[i], 0).u32;
+
+            UInt32 pitches[4] = { 0 };
+            pitches[0] = gbm_bo_get_stride_for_plane(data->connectorBOs[i], 0);
+
+            UInt32 offsets[4] = { 0 };
+            offsets[0] = gbm_bo_get_offset(data->connectorBOs[i], 0);
+
+            UInt64 mods[4] = { 0 };
+            mods[0] = connector->currentFormat.modifier;
+
+            ret = drmModeAddFB2WithModifiers(connector->device->fd,
+                               connector->currentMode->info.hdisplay,
+                               connector->currentMode->info.vdisplay,
+                               connector->currentFormat.format,
+                               boHandles,
+                               pitches,
+                               offsets,
+                               mods,
+                               &data->connectorDRMFramebuffers[i],
+                               DRM_MODE_FB_MODIFIERS);
+
+            if (ret == 0)
+                goto skipLegacy;
+        }
+
         ret = drmModeAddFB(connector->device->fd,
                            connector->currentMode->info.hdisplay,
                            connector->currentMode->info.vdisplay,
@@ -343,6 +350,8 @@ static UInt8 createDRMFramebuffers(SRMConnector *connector)
             return 0;
         }
 
+        skipLegacy:
+
         data->buffers[i] = srmBufferCreateFromGBM(connector->device->core, data->connectorBOs[i]);
 
         if (!data->buffers[i])
@@ -442,6 +451,7 @@ static UInt8 render(SRMConnector *connector)
 static UInt8 flipPage(SRMConnector *connector)
 {
     RenderModeData *data = (RenderModeData*)connector->renderData;
+
     swapBuffers(connector, connector->device->eglDisplay, data->connectorEGLSurface);
     gbm_surface_lock_front_buffer(data->connectorGBMSurface);
 
@@ -449,6 +459,7 @@ static UInt8 flipPage(SRMConnector *connector)
 
     data->currentBufferIndex = nextBufferIndex(connector);
     gbm_surface_release_buffer(data->connectorGBMSurface, data->connectorBOs[data->currentBufferIndex]);
+
     connector->interface->pageFlipped(connector, connector->interfaceData);
     return 1;
 }
@@ -480,6 +491,9 @@ static void uninitialize(SRMConnector *connector)
 
 static UInt8 initialize(SRMConnector *connector)
 {
+    connector->allowModifiers = 1;
+
+retry:
     if (!eglBindAPI(EGL_OPENGL_ES_API))
     {
         SRMError("Failed to bind GLES API for device %s connector %d (ITSELF MODE).",
@@ -512,11 +526,21 @@ static UInt8 initialize(SRMConnector *connector)
     return 1;
 
 fail:
+    uninitialize(connector);
+
+    if (connector->allowModifiers)
+    {
+        SRMError("Failed to initialize device %s connector %d with explicit modifiers, falling back to implicit modifiers (ITSELF MODE).",
+                 connector->device->name,
+                 connector->id);
+        connector->allowModifiers = 0;
+        goto retry;
+    }
+
     SRMError("Failed to initialize render mode ITSELF for device %s connector %d.",
              connector->device->name,
              connector->id);
 
-    uninitialize(connector);
     return 0;
 }
 
@@ -575,6 +599,12 @@ static void resumeRendering(SRMConnector *connector)
     srmRenderModeCommonResumeRendering(connector, data->connectorDRMFramebuffers[data->currentBufferIndex]);
 }
 
+static EGLContext getEGLContext(SRMConnector *connector)
+{
+    RenderModeData *data = (RenderModeData*)connector->renderData;
+    return data->connectorEGLContext;
+}
+
 void srmRenderModeItselfSetInterface(SRMConnector *connector)
 {
     connector->renderInterface.initialize = &initialize;
@@ -584,6 +614,7 @@ void srmRenderModeItselfSetInterface(SRMConnector *connector)
     connector->renderInterface.getCurrentBufferIndex = &getCurrentBufferIndex;
     connector->renderInterface.getBuffersCount = &getBuffersCount;
     connector->renderInterface.getBuffer = &getBuffer;
+    connector->renderInterface.getEGLContext = &getEGLContext;
     connector->renderInterface.uninitialize = &uninitialize;
     connector->renderInterface.pause = &pauseRendering;
     connector->renderInterface.resume = &resumeRendering;
